# coding=utf-8
from collections.abc import MutableMapping
from typing import Any, Callable, Optional, TypeVar

from corehttp.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from corehttp.rest import HttpRequest, HttpResponse
from corehttp.runtime import PipelineClient
from corehttp.runtime.pipeline import PipelineResponse
from corehttp.utils import case_insensitive_dict

from ..._configuration import CollectionFormatClientConfiguration
from ..._utils.serialization import Deserializer, Serializer

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_header_csv_request(*, colors: list[str], **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    # Construct URL
    _url = "/parameters/collection-format/header/csv"

    # Construct headers
    _headers["colors"] = _SERIALIZER.header("colors", colors, "[str]", div=",")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


class HeaderOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~parameters.collectionformat.CollectionFormatClient`'s
        :attr:`header` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: PipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: CollectionFormatClientConfiguration = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    def csv(self, *, colors: list[str], **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """csv.

        :keyword colors: Possible values for colors are [blue,red,green]. Required.
        :paramtype colors: list[str]
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_header_csv_request(
            colors=colors,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore



# coding=utf-8

from enum import Enum
from corehttp.utils import CaseInsensitiveEnumMeta


class BulletPointsEnum(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """This tests really long bullet points in enum documentation to see how wrapping and formatting
    are handled. This should wrap around correctly and maintain proper indentation for each line.

    * Simple bullet point. This bullet point is going to be very long to test how text wrapping is
    handled in bullet points within documentation comments. It should properly indent the wrapped
    lines.
    * Another bullet point with **bold text**. This bullet point is also intentionally long to see
    how the formatting is preserved when the text wraps onto multiple lines in the generated
    documentation.
    * Third bullet point with *italic text*. Similar to the previous points, this one is extended
    to ensure that the wrapping and formatting are correctly applied in the output.
    * Complex bullet point with **bold** and *italic* combined. This bullet point combines both
    bold and italic formatting and is long enough to test the wrapping behavior in such cases.
    * **Bold bullet point**: A bullet point that is entirely bolded. This point is also made
    lengthy to observe how the bold formatting is maintained across wrapped lines.
    * *Italic bullet point*: A bullet point that is entirely italicized. This final point is
    extended to verify that italic formatting is correctly applied even when the text spans
    multiple lines.
    """

    SIMPLE = "Simple"
    """Simple bullet point. This line is intentionally long to test text wrapping in bullet points
    within enum documentation comments. It should properly indent the wrapped lines.
    
    * One: one. This line is intentionally long to test text wrapping in bullet points within enum
    documentation comments. It should properly indent the wrapped lines.
    * Two: two. This line is intentionally long to test text wrapping in bullet points within enum
    documentation comments. It should properly indent the wrapped lines."""
    BOLD = "Bold"
    """Bullet point with **bold text**. This line is intentionally long to test text wrapping in
    bullet points within enum documentation comments. It should properly indent the wrapped lines.
    
    * **One**: one. This line is intentionally long to test text wrapping in bullet points within
    enum documentation comments. It should properly indent the wrapped lines.
    * **Two**: two. This line is intentionally long to test text wrapping in bullet points within
    enum documentation comments. It should properly indent the wrapped lines."""
    ITALIC = "Italic"
    """Bullet point with *italic text*. This line is intentionally long to test text wrapping in
    bullet points within enum documentation comments. It should properly indent the wrapped lines.
    
    * *One*: one. This line is intentionally long to test text wrapping in bullet points within
    enum documentation comments. It should properly indent the wrapped lines.
    * *Two*: two. This line is intentionally long to test text wrapping in bullet points within
    enum documentation comments. It should properly indent the wrapped lines."""



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.
"""Contract tests for GET /responses/{response_id} endpoint behavior."""

from __future__ import annotations

import asyncio
import json
from typing import Any

from starlette.testclient import TestClient

from azure.ai.agentserver.responses import ResponsesAgentServerHost


def _noop_response_handler(request: Any, context: Any, cancellation_signal: Any):
    """Minimal handler used to wire the hosting surface in contract tests."""

    async def _events():
        if False:  # pragma: no cover - required to keep async-generator shape.
            yield None

    return _events()


def _build_client() -> TestClient:
    app = ResponsesAgentServerHost()
    app.response_handler(_noop_response_handler)
    return TestClient(app)


def _collect_replay_events(response: Any) -> list[dict[str, Any]]:
    events: list[dict[str, Any]] = []
    current_type: str | None = None
    current_data: str | None = None

    for line in response.iter_lines():
        if not line:
            if current_type is not None:
                payload = json.loads(current_data) if current_data else {}
                events.append({"type": current_type, "data": payload})
            current_type = None
            current_data = None
            continue

        if line.startswith("event:"):
            current_type = line.split(":", 1)[1].strip()
        elif line.startswith("data:"):
            current_data = line.split(":", 1)[1].strip()

    if current_type is not None:
        payload = json.loads(current_data) if current_data else {}
        events.append({"type": current_type, "data": payload})

    return events


def _create_streaming_and_get_response_id(client: TestClient) -> str:
    with client.stream(
        "POST",
        "/responses",
        json={
            "model": "gpt-4o-mini",
            "input": "hello",
            "stream": True,
            "store": True,
            "background": False,
        },
    ) as create_response:
        assert create_response.status_code == 200
        assert create_response.headers.get("content-type", "").startswith("text/event-stream")
        events = _collect_replay_events(create_response)

    assert events, "Expected streaming create to emit at least one event"
    response_id = events[0]["data"]["response"].get("id")
    assert isinstance(response_id, str)
    return response_id


def _create_background_streaming_and_get_response_id(client: TestClient) -> str:
    with client.stream(
        "POST",
        "/responses",
        json={
            "model": "gpt-4o-mini",
            "input": "hello",
            "stream": True,
            "store": True,
            "background": True,
        },
    ) as create_response:
        assert create_response.status_code == 200
        assert create_response.headers.get("content-type", "").startswith("text/event-stream")
        events = _collect_replay_events(create_response)

    assert events, "Expected background streaming create to emit at least one event"
    response_id = events[0]["data"]["response"].get("id")
    assert isinstance(response_id, str)
    return response_id


def test_get__returns_latest_snapshot_for_existing_response() -> None:
    client = _build_client()

    create_response = client.post(
        "/responses",
        json={
            "model": "gpt-4o-mini",
            "input": "hello",
            "stream": False,
            "store": True,
            "background": False,
        },
    )
    assert create_response.status_code == 200
    response_id = create_response.json()["id"]

    get_response = client.get(f"/responses/{response_id}")
    assert get_response.status_code == 200
    payload = get_response.json()
    assert payload.get("id") == response_id
    assert payload.get("response_id") == response_id
    assert payload.get("object") == "response"
    assert isinstance(payload.get("agent_reference"), dict)
    assert payload["agent_reference"].get("type") == "agent_reference"
    assert payload.get("status") in {"queued", "in_progress", "completed", "failed", "incomplete", "cancelled"}
    assert payload.get("model") == "gpt-4o-mini"
    assert "sequence_number" not in payload


def test_get__returns_404_for_unknown_response_id() -> None:
    from azure.ai.agentserver.responses._id_generator import IdGenerator

    client = _build_client()
    unknown_id = IdGenerator.new_response_id()

    get_response = client.get(f"/responses/{unknown_id}")
    assert get_response.status_code == 404
    payload = get_response.json()
    assert isinstance(payload.get("error"), dict)
    assert payload["error"].get("type") == "invalid_request_error"
    assert payload["error"].get("code") == "invalid_request_error"
    # 404 message must reference the requested response ID
    error_message = payload["error"].get("message", "")
    assert unknown_id in error_message, f"404 error message should reference the response ID, got: {error_message!r}"


def test_get__returns_snapshot_for_stored_non_background_stream_response_after_completion() -> None:
    client = _build_client()

    response_id = _create_streaming_and_get_response_id(client)

    get_response = client.get(f"/responses/{response_id}")
    assert get_response.status_code == 200
    payload = get_response.json()
    assert payload.get("id") == response_id
    assert payload.get("status") in {"completed", "failed", "incomplete", "cancelled"}


def test_get_replay__rejects_request_when_replay_preconditions_are_not_met() -> None:
    client = _build_client()

    response_id = _create_streaming_and_get_response_id(client)

    replay_response = client.get(f"/responses/{response_id}?stream=true")
    assert replay_response.status_code == 400
    payload = replay_response.json()
    assert isinstance(payload.get("error"), dict)
    assert payload["error"].get("type") == "invalid_request_error"
    assert payload["error"].get("code") == "invalid_request_error"
    assert payload["error"].get("param") == "stream"
    error_message = payload["error"].get("message", "")
    assert "background=true" in error_message, (
        f"SSE replay rejection for non-bg response must mention 'background=true', got: {error_message!r}"
    )


def test_get_replay__rejects_invalid_starting_after_cursor_type() -> None:
    client = _build_client()

    response_id = _create_background_streaming_and_get_response_id(client)

    replay_response = client.get(f"/responses/{response_id}?stream=true&starting_after=not-an-int")
    assert replay_response.status_code == 400
    payload = replay_response.json()
    assert payload["error"].get("type") == "invalid_request_error"
    assert payload["error"].get("param") == "starting_after"


def test_get_replay__starting_after_returns_events_after_cursor() -> None:
    client = _build_client()

    response_id = _create_background_streaming_and_get_response_id(client)

    with client.stream("GET", f"/responses/{response_id}?stream=true&starting_after=0") as replay_response:
        assert replay_response.status_code == 200
        assert replay_response.headers.get("content-type", "").startswith("text/event-stream")
        replay_events = _collect_replay_events(replay_response)

    assert replay_events, "Expected replay stream to include events after cursor"
    sequence_numbers = [event["data"].get("sequence_number") for event in replay_events]
    assert all(isinstance(sequence_number, int) for sequence_number in sequence_numbers)
    assert min(sequence_numbers) > 0
    terminal_events = {
        "response.completed",
        "response.failed",
        "response.incomplete",
    }
    assert any(event["type"] in terminal_events for event in replay_events)


def test_get_replay__rejects_bg_non_stream_response() -> None:
    """B2 — SSE replay on bg+non-stream after eviction → 400 with combined message.

    After eager eviction the persisted response doesn't carry the stream mode
    flag, so the server cannot distinguish bg+non-stream from bg+stream with
    expired TTL.  The error uses a combined message matching .NET's SseReplayResult.
    """
    client = _build_client()

    create_response = client.post(
        "/responses",
        json={
            "model": "gpt-4o-mini",
            "input": "hello",
            "stream": False,
            "store": True,
            "background": True,
        },
    )
    assert create_response.status_code == 200
    response_id = create_response.json()["id"]

    replay_response = client.get(f"/responses/{response_id}?stream=true")
    assert replay_response.status_code == 400
    payload = replay_response.json()
    assert payload["error"]["type"] == "invalid_request_error"
    assert payload["error"].get("code") == "invalid_request_error"
    error_message = payload["error"].get("message", "")
    assert "stream=true" in error_message, f"SSE replay rejection must mention 'stream=true', got: {error_message!r}"
    assert payload["error"].get("param") == "stream"


# ══════════════════════════════════════════════════════════
# B5: SSE replay rejection message text
# ══════════════════════════════════════════════════════════


def test_get_replay__rejection_message_hints_at_background_true() -> None:
    """B5 — SSE replay rejection error message contains 'background=true' hint.

    Clients should know how to fix their request.
    """
    client = _build_client()

    create_response = client.post(
        "/responses",
        json={
            "model": "gpt-4o-mini",
            "input": "hello",
            "stream": False,
            "store": True,
            "background": False,
        },
    )
    assert create_response.status_code == 200
    response_id = create_response.json()["id"]

    replay_response = client.get(f"/responses/{response_id}?stream=true")
    assert replay_response.status_code == 400
    payload = replay_response.json()
    error_message = payload["error"].get("message", "")
    assert "background=true" in error_message, (
        f"Error message should hint at 'background=true' to guide the client, but got: {error_message!r}"
    )
    assert payload["error"].get("code") == "invalid_request_error"
    assert payload["error"].get("param") == "stream"


# ════════════════════════════════════════════════════════
# N-6: GET ?stream=true SSE response headers
# ════════════════════════════════════════════════════════


def test_get_replay__sse_response_headers_are_correct() -> None:
    """SSE headers contract — GET ?stream=true replay must return required SSE response headers."""
    client = _build_client()

    response_id = _create_background_streaming_and_get_response_id(client)

    with client.stream("GET", f"/responses/{response_id}?stream=true") as replay_response:
        assert replay_response.status_code == 200
        headers = replay_response.headers

    content_type = headers.get("content-type", "")
    assert "text/event-stream" in content_type, (
        f"SSE replay Content-Type must be text/event-stream, got: {content_type!r}"
    )
    assert headers.get("cache-control") == "no-cache", (
        f"SSE replay Cache-Control must be no-cache, got: {headers.get('cache-control')!r}"
    )
    assert headers.get("connection", "").lower() == "keep-alive", (
        f"SSE replay Connection must be keep-alive, got: {headers.get('connection')!r}"
    )
    assert headers.get("x-accel-buffering") == "no", (
        f"SSE replay X-Accel-Buffering must be no, got: {headers.get('x-accel-buffering')!r}"
    )


# ══════════════════════════════════════════════════════════
# Task 4.2 — _finalize_bg_stream / _finalize_non_bg_stream
# ══════════════════════════════════════════════════════════


def test_c2_sync_stream_stored_get_returns_200() -> None:
    """T1 — store=True, bg=False, stream=True: POST then GET returns HTTP 200.

    _finalize_non_bg_stream must register a ResponseExecution so that the
    subsequent GET can find the stored non-background stream response.
    """
    client = _build_client()

    with client.stream(
        "POST",
        "/responses",
        json={"model": "gpt-4o-mini", "input": "hello", "stream": True, "store": True, "background": False},
    ) as create_response:
        assert create_response.status_code == 200
        events = _collect_replay_events(create_response)

    assert events, "Expected at least one SSE event"
    response_id = events[0]["data"]["response"].get("id")
    assert isinstance(response_id, str)

    get_response = client.get(f"/responses/{response_id}")
    assert get_response.status_code == 200, (
        f"_finalize_non_bg_stream must persist the record so GET returns 200, got {get_response.status_code}"
    )
    payload = get_response.json()
    assert payload.get("status") in {"completed", "failed", "incomplete", "cancelled"}, (
        f"Non-bg stored stream must be terminal after POST completes, got status={payload.get('status')!r}"
    )


def test_c4_bg_stream_get_sse_replay() -> None:
    """T2 — store=True, bg=True, stream=True: POST complete, then GET ?stream=true returns SSE replay.

    _finalize_bg_stream must complete the subject so that the subsequent
    replay GET can iterate the historical events to completion.
    """
    client = _build_client()

    with client.stream(
        "POST",
        "/responses",
        json={"model": "gpt-4o-mini", "input": "hello", "stream": True, "store": True, "background": True},
    ) as create_response:
        assert create_response.status_code == 200
        create_events = _collect_replay_events(create_response)

    assert create_events, "Expected at least one SSE event from POST"
    response_id = create_events[0]["data"]["response"].get("id")
    assert isinstance(response_id, str)

    with client.stream("GET", f"/responses/{response_id}?stream=true") as replay_response:
        assert replay_response.status_code == 200, (
            f"bg+stream GET ?stream=true must return 200, got {replay_response.status_code}"
        )
        assert replay_response.headers.get("content-type", "").startswith("text/event-stream")
        replay_events = _collect_replay_events(replay_response)

    assert replay_events, "Expected at least one event in SSE replay"
    replay_types = [e["type"] for e in replay_events]
    terminal_types = {"response.completed", "response.failed", "response.incomplete"}
    assert any(t in terminal_types for t in replay_types), (
        f"SSE replay must include a terminal event, got: {replay_types}"
    )
    # Replay must start from the beginning (response.created should be present)
    assert "response.created" in replay_types, f"SSE replay must include response.created, got: {replay_types}"


def test_c6_non_stored_stream_no_get() -> None:
    """T3 — store=False, bg=False, stream=True: GET returns HTTP 404.

    _finalize_non_bg_stream must NOT register the execution record when
    store=False, so a subsequent GET returns 404 (B16 / C6 contract).
    """
    client = _build_client()

    with client.stream(
        "POST",
        "/responses",
        json={"model": "gpt-4o-mini", "input": "hello", "stream": True, "store": False, "background": False},
    ) as create_response:
        assert create_response.status_code == 200
        create_events = _collect_replay_events(create_response)

    assert create_events, "Expected at least one SSE event from POST"
    response_id = create_events[0]["data"]["response"].get("id")
    assert isinstance(response_id, str)

    get_response = client.get(f"/responses/{response_id}")
    assert get_response.status_code == 404, (
        f"store=False stream response must not be retrievable via GET (C6), got {get_response.status_code}"
    )


def test_bg_stream_cancelled_subject_completed() -> None:
    """T4 — bg+stream response cancelled mid-stream: subject.complete() is called, no hang.

    _finalize_bg_stream must call subject.complete() even when the record's
    status is 'cancelled', so that live replay subscribers can exit cleanly.
    """
    from azure.ai.agentserver.responses._id_generator import IdGenerator
    from tests._helpers import poll_until

    gate_started: list[bool] = []

    def _blocking_bg_stream_handler(request: Any, context: Any, cancellation_signal: Any):
        async def _events():
            yield {"type": "response.created", "response": {"status": "in_progress", "output": []}}
            gate_started.append(True)
            # Block until cancelled
            while not cancellation_signal.is_set():
                import asyncio as _asyncio

                await _asyncio.sleep(0.01)

        return _events()

    import threading

    _app = ResponsesAgentServerHost()
    _app.response_handler(_blocking_bg_stream_handler)  # type: ignore[arg-type]  # yields raw dicts to test coercion
    app = _app

    response_id = IdGenerator.new_response_id()
    stream_events_received: list[str] = []
    stream_done = threading.Event()

    def _stream_thread() -> None:
        from starlette.testclient import TestClient as _TC

        _client = _TC(app)
        with _client.stream(
            "POST",
            "/responses",
            json={
                "response_id": response_id,
                "model": "gpt-4o-mini",
                "input": "hello",
                "stream": True,
                "store": True,
                "background": True,
            },
        ) as resp:
            for line in resp.iter_lines():
                stream_events_received.append(line)
        stream_done.set()

    t = threading.Thread(target=_stream_thread, daemon=True)
    t.start()

    # Wait for handler to start
    ok, _ = poll_until(
        lambda: bool(gate_started),
        timeout_s=5.0,
        interval_s=0.02,
        label="wait for bg stream handler to start",
    )
    assert ok, "Handler did not start within timeout"

    # Cancel the response
    from starlette.testclient import TestClient as _TC2

    _cancel_client = _TC2(app)
    cancel_resp = _cancel_client.post(f"/responses/{response_id}/cancel")
    assert cancel_resp.status_code == 200

    # The SSE stream should terminate (subject.complete() unblocks the iterator)
    assert stream_done.wait(timeout=5.0), (
        "_finalize_bg_stream must call subject.complete() so SSE stream terminates after cancel"
    )
    t.join(timeout=1.0)


# ---------------------------------------------------------------------------
# Missing protocol parity tests (ported from GetResponseProtocolTests)
# ---------------------------------------------------------------------------


def _cancellable_bg_handler(request: Any, context: Any, cancellation_signal: Any):
    """Handler that blocks until cancelled — keeps bg response in_progress."""

    async def _events():
        yield {
            "type": "response.created",
            "response": {"status": "in_progress", "output": []},
        }
        while not cancellation_signal.is_set():
            await asyncio.sleep(0.01)

    return _events()


def test_get__in_progress_bg_response_returns_200() -> None:
    """GET on a background response that is still in_progress returns 200 with status in_progress."""
    app = ResponsesAgentServerHost()
    app.response_handler(_cancellable_bg_handler)  # type: ignore[arg-type]  # yields raw dicts to test coercion
    client = TestClient(app)

    create = client.post(
        "/responses",
        json={"model": "test", "input": "hello", "stream": False, "store": True, "background": True},
    )
    assert create.status_code == 200
    response_id = create.json()["id"]

    get = client.get(f"/responses/{response_id}")
    assert get.status_code == 200
    assert get.json()["status"] == "in_progress"

    # Clean up
    client.post(f"/responses/{response_id}/cancel")


def test_get__cancelled_bg_returns_200_with_cancelled_status() -> None:
    """GET on a cancelled background response returns 200 with status=cancelled and empty output."""
    app = ResponsesAgentServerHost()
    app.response_handler(_cancellable_bg_handler)  # type: ignore[arg-type]  # yields raw dicts to test coercion
    client = TestClient(app)

    create = client.post(
        "/responses",
        json={"model": "test", "input": "hello", "stream": False, "store": True, "background": True},
    )
    assert create.status_code == 200
    response_id = create.json()["id"]

    cancel = client.post(f"/responses/{response_id}/cancel")
    assert cancel.status_code == 200

    get = client.get(f"/responses/{response_id}")
    assert get.status_code == 200
    payload = get.json()
    assert payload["status"] == "cancelled"
    assert payload.get("output", []) == [], "Cancelled response must have 0 output items"


def test_get__sse_replay_starting_after_max_returns_no_events() -> None:
    """SSE replay with starting_after >= max sequence number returns an empty event stream."""
    client = _build_client()
    response_id = _create_background_streaming_and_get_response_id(client)

    # starting_after=9999 — way beyond any sequence in a simple handler
    with client.stream(
        "GET",
        f"/responses/{response_id}?stream=true&starting_after=9999",
    ) as replay:
        assert replay.status_code == 200
        events = _collect_replay_events(replay)

    assert events == [], "Replay with starting_after >= max seq must return 0 events"


def test_get__sse_replay_store_false_returns_404() -> None:
    """SSE replay on a store=false response returns 404."""
    client = _build_client()

    create = client.post(
        "/responses",
        json={"model": "test", "input": "hello", "stream": False, "store": False},
    )
    assert create.status_code == 200
    response_id = create.json()["id"]

    with client.stream("GET", f"/responses/{response_id}?stream=true") as replay:
        assert replay.status_code == 404


def test_get__sse_replay_unknown_id_returns_404_with_error_shape() -> None:
    """SSE replay on a completely unknown response ID returns 404 with proper error envelope."""
    client = _build_client()

    from azure.ai.agentserver.responses._id_generator import IdGenerator

    unknown_id = IdGenerator.new_response_id()
    replay_response = client.get(f"/responses/{unknown_id}?stream=true")
    assert replay_response.status_code == 404
    payload = replay_response.json()
    assert isinstance(payload.get("error"), dict)
    assert payload["error"].get("type") == "invalid_request_error"
    assert payload["error"].get("code") == "invalid_request_error"


def test_get__stream_false_returns_json_snapshot() -> None:
    """Explicit ?stream=false returns a JSON snapshot, not SSE."""
    client = _build_client()
    response_id = _create_background_streaming_and_get_response_id(client)

    get = client.get(f"/responses/{response_id}?stream=false")
    assert get.status_code == 200
    assert get.headers.get("content-type", "").startswith("application/json")
    assert get.json()["id"] == response_id


def test_get__sse_replay_has_correct_sequence_numbers() -> None:
    """SSE replay produces monotonically increasing sequence numbers starting from 0."""
    client = _build_client()
    response_id = _create_background_streaming_and_get_response_id(client)

    with client.stream("GET", f"/responses/{response_id}?stream=true") as replay:
        assert replay.status_code == 200
        events = _collect_replay_events(replay)

    assert len(events) >= 2, "Expected at least 2 replayed SSE events"
    seq_nums = [e["data"].get("sequence_number") for e in events]
    assert seq_nums[0] == 0, "First sequence_number must be 0"
    for i in range(1, len(seq_nums)):
        assert seq_nums[i] > seq_nums[i - 1], (
            f"Sequence numbers not monotonically increasing at index {i}: {seq_nums[i - 1]} → {seq_nums[i]}"
        )


def test_get__accept_sse_without_stream_true_returns_json_snapshot() -> None:
    """Accept: text/event-stream WITHOUT ?stream=true returns JSON snapshot — Accept header is NOT a trigger for SSE.

    Ported from GetResponseProtocolTests.GET_WithAcceptSse_WithoutStreamTrue_Returns200_JsonSnapshot.
    """
    client = _build_client()
    response_id = _create_background_streaming_and_get_response_id(client)

    get = client.get(
        f"/responses/{response_id}",
        headers={"Accept": "text/event-stream"},
    )
    assert get.status_code == 200
    content_type = get.headers.get("content-type", "")
    assert content_type.startswith("application/json"), (
        f"Expected application/json when Accept: text/event-stream but no ?stream=true, got {content_type!r}"
    )
    assert get.json()["id"] == response_id


def test_get__store_false_returns_404() -> None:
    """GET on a store=false response returns 404.

    Ported from GetResponseProtocolTests.GET_StoreFalse_Returns404.
    """
    client = _build_client()

    create = client.post(
        "/responses",
        json={"model": "test", "input": "hello", "stream": False, "store": False},
    )
    assert create.status_code == 200
    response_id = create.json()["id"]

    get = client.get(f"/responses/{response_id}")
    assert get.status_code == 404



# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from collections.abc import MutableMapping
from io import IOBase
import json
from typing import Any, Callable, IO, Optional, TypeVar, Union, overload

from azure.core import PipelineClient
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.rest import HttpRequest, HttpResponse
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict

from .. import models as _models
from .._configuration import NamingClientConfiguration
from .._utils.model_base import SdkJSONEncoder
from .._utils.serialization import Deserializer, Serializer
from .._utils.utils import ClientMixinABC

JSON = MutableMapping[str, Any]
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_property_client_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    # Construct URL
    _url = "/client/naming/property/client"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_property_language_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    # Construct URL
    _url = "/client/naming/property/language"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_property_compatible_with_encoded_name_request(**kwargs: Any) -> HttpRequest:  # pylint: disable=name-too-long
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    # Construct URL
    _url = "/client/naming/property/compatible-with-encoded-name"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_header_request_request(*, client_name: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    # Construct URL
    _url = "/client/naming/header"

    # Construct headers
    _headers["default-name"] = _SERIALIZER.header("client_name", client_name, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_header_response_request(**kwargs: Any) -> HttpRequest:
    # Construct URL
    _url = "/client/naming/header"

    return HttpRequest(method="GET", url=_url, **kwargs)


def build_model_client_client_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    # Construct URL
    _url = "/client/naming/model/client"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_model_client_language_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    # Construct URL
    _url = "/client/naming/model/language"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_union_enum_union_enum_name_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type: str = kwargs.pop("content_type")
    # Construct URL
    _url = "/client/naming/union-enum/union-enum-name"

    # Construct headers
    _headers["content-type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_union_enum_union_enum_member_name_request(**kwargs: Any) -> HttpRequest:  # pylint: disable=name-too-long
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type: str = kwargs.pop("content_type")
    # Construct URL
    _url = "/client/naming/union-enum/union-enum-member-name"

    # Construct headers
    _headers["content-type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_naming_client_name_request(**kwargs: Any) -> HttpRequest:
    # Construct URL
    _url = "/client/naming/operation"

    return HttpRequest(method="POST", url=_url, **kwargs)


def build_naming_parameter_request(*, client_name: str, **kwargs: Any) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/client/naming/parameter"

    # Construct parameters
    _params["defaultName"] = _SERIALIZER.query("client_name", client_name, "str")

    return HttpRequest(method="POST", url=_url, params=_params, **kwargs)


class PropertyOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~client.naming.main.NamingClient`'s
        :attr:`property` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: PipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: NamingClientConfiguration = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @overload
    def client(self, body: _models.ClientNameModel, *, content_type: str = "application/json", **kwargs: Any) -> None:
        """client.

        :param body: Required.
        :type body: ~client.naming.main.models.ClientNameModel
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def client(self, body: JSON, *, content_type: str = "application/json", **kwargs: Any) -> None:
        """client.

        :param body: Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def client(self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any) -> None:
        """client.

        :param body: Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def client(  # pylint: disable=inconsistent-return-statements
        self, body: Union[_models.ClientNameModel, JSON, IO[bytes]], **kwargs: Any
    ) -> None:
        """client.

        :param body: Is one of the following types: ClientNameModel, JSON, IO[bytes] Required.
        :type body: ~client.naming.main.models.ClientNameModel or JSON or IO[bytes]
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[None] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_property_client_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @overload
    def language(
        self, body: _models.LanguageClientNameModel, *, content_type: str = "application/json", **kwargs: Any
    ) -> None:
        """language.

        :param body: Required.
        :type body: ~client.naming.main.models.LanguageClientNameModel
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def language(self, body: JSON, *, content_type: str = "application/json", **kwargs: Any) -> None:
        """language.

        :param body: Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def language(self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any) -> None:
        """language.

        :param body: Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def language(  # pylint: disable=inconsistent-return-statements
        self, body: Union[_models.LanguageClientNameModel, JSON, IO[bytes]], **kwargs: Any
    ) -> None:
        """language.

        :param body: Is one of the following types: LanguageClientNameModel, JSON, IO[bytes] Required.
        :type body: ~client.naming.main.models.LanguageClientNameModel or JSON or IO[bytes]
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[None] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_property_language_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @overload
    def compatible_with_encoded_name(
        self, body: _models.ClientNameAndJsonEncodedNameModel, *, content_type: str = "application/json", **kwargs: Any
    ) -> None:
        """compatible_with_encoded_name.

        :param body: Required.
        :type body: ~client.naming.main.models.ClientNameAndJsonEncodedNameModel
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def compatible_with_encoded_name(
        self, body: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> None:
        """compatible_with_encoded_name.

        :param body: Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def compatible_with_encoded_name(
        self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> None:
        """compatible_with_encoded_name.

        :param body: Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def compatible_with_encoded_name(  # pylint: disable=inconsistent-return-statements
        self, body: Union[_models.ClientNameAndJsonEncodedNameModel, JSON, IO[bytes]], **kwargs: Any
    ) -> None:
        """compatible_with_encoded_name.

        :param body: Is one of the following types: ClientNameAndJsonEncodedNameModel, JSON, IO[bytes]
         Required.
        :type body: ~client.naming.main.models.ClientNameAndJsonEncodedNameModel or JSON or IO[bytes]
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[None] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_property_compatible_with_encoded_name_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore


class HeaderOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~client.naming.main.NamingClient`'s
        :attr:`header` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: PipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: NamingClientConfiguration = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def request(self, *, client_name: str, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """request.

        :keyword client_name: Required.
        :paramtype client_name: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_header_request_request(
            client_name=client_name,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @distributed_trace
    def response(self, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """response.

        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_header_response_request(
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        response_headers = {}
        response_headers["default-name"] = self._deserialize("str", response.headers.get("default-name"))

        if cls:
            return cls(pipeline_response, None, response_headers)  # type: ignore


class ModelClientOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~client.naming.main.NamingClient`'s
        :attr:`model_client` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: PipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: NamingClientConfiguration = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @overload
    def client(self, body: _models.ClientModel, *, content_type: str = "application/json", **kwargs: Any) -> None:
        """client.

        :param body: Required.
        :type body: ~client.naming.main.models.ClientModel
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def client(self, body: JSON, *, content_type: str = "application/json", **kwargs: Any) -> None:
        """client.

        :param body: Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def client(self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any) -> None:
        """client.

        :param body: Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def client(  # pylint: disable=inconsistent-return-statements
        self, body: Union[_models.ClientModel, JSON, IO[bytes]], **kwargs: Any
    ) -> None:
        """client.

        :param body: Is one of the following types: ClientModel, JSON, IO[bytes] Required.
        :type body: ~client.naming.main.models.ClientModel or JSON or IO[bytes]
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[None] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_model_client_client_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @overload
    def language(self, body: _models.PythonModel, *, content_type: str = "application/json", **kwargs: Any) -> None:
        """language.

        :param body: Required.
        :type body: ~client.naming.main.models.PythonModel
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def language(self, body: JSON, *, content_type: str = "application/json", **kwargs: Any) -> None:
        """language.

        :param body: Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def language(self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any) -> None:
        """language.

        :param body: Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def language(  # pylint: disable=inconsistent-return-statements
        self, body: Union[_models.PythonModel, JSON, IO[bytes]], **kwargs: Any
    ) -> None:
        """language.

        :param body: Is one of the following types: PythonModel, JSON, IO[bytes] Required.
        :type body: ~client.naming.main.models.PythonModel or JSON or IO[bytes]
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[None] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_model_client_language_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore


class UnionEnumOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~client.naming.main.NamingClient`'s
        :attr:`union_enum` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: PipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: NamingClientConfiguration = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def union_enum_name(  # pylint: disable=inconsistent-return-statements
        self, body: Union[str, _models.ClientExtensibleEnum], **kwargs: Any
    ) -> None:
        """union_enum_name.

        :param body: "value1" Required.
        :type body: str or ~client.naming.main.models.ClientExtensibleEnum
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: str = kwargs.pop("content_type", _headers.pop("content-type", "application/json"))
        cls: ClsType[None] = kwargs.pop("cls", None)

        _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_union_enum_union_enum_name_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @distributed_trace
    def union_enum_member_name(  # pylint: disable=inconsistent-return-statements
        self, body: Union[str, _models.ExtensibleEnum], **kwargs: Any
    ) -> None:
        """union_enum_member_name.

        :param body: Known values are: "value1" and "value2". Required.
        :type body: str or ~client.naming.main.models.ExtensibleEnum
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: str = kwargs.pop("content_type", _headers.pop("content-type", "application/json"))
        cls: ClsType[None] = kwargs.pop("cls", None)

        _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_union_enum_union_enum_member_name_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore


class _NamingClientOperationsMixin(
    ClientMixinABC[PipelineClient[HttpRequest, HttpResponse], NamingClientConfiguration]
):

    @distributed_trace
    def client_name(self, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """client_name.

        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_naming_client_name_request(
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @distributed_trace
    def parameter(self, *, client_name: str, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """parameter.

        :keyword client_name: Required.
        :paramtype client_name: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_naming_parameter_request(
            client_name=client_name,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore



# coding=utf-8
from collections.abc import MutableMapping
from typing import Any, Callable, Optional, TypeVar

from corehttp.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from corehttp.rest import AsyncHttpResponse, HttpRequest
from corehttp.runtime import AsyncPipelineClient
from corehttp.runtime.pipeline import PipelineResponse

from ......_utils.serialization import Deserializer, Serializer
from ......aio._configuration import RoutesClientConfiguration
from ...operations._operations import (
    build_path_parameters_matrix_expansion_explode_array_request,
    build_path_parameters_matrix_expansion_explode_primitive_request,
    build_path_parameters_matrix_expansion_explode_record_request,
)

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, dict[str, Any]], Any]]


class PathParametersMatrixExpansionExplodeOperations:  # pylint: disable=name-too-long
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~routes.aio.RoutesClient`'s
        :attr:`explode` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: AsyncPipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: RoutesClientConfiguration = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    async def primitive(self, param: str, **kwargs: Any) -> None:
        """primitive.

        :param param: Required.
        :type param: str
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_path_parameters_matrix_expansion_explode_primitive_request(
            param=param,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = await self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    async def array(self, param: list[str], **kwargs: Any) -> None:
        """array.

        :param param: Required.
        :type param: list[str]
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_path_parameters_matrix_expansion_explode_array_request(
            param=param,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = await self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    async def record(self, param: dict[str, int], **kwargs: Any) -> None:
        """record.

        :param param: Required.
        :type param: dict[str, int]
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_path_parameters_matrix_expansion_explode_record_request(
            param=param,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = await self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import pytest
from devtools_testutils.aio import recorded_by_proxy_async
from devtools_testutils import AzureRecordedTestCase, RecordedTransport
from test_base import servicePreparer
from sample_executor import (
    AsyncSampleExecutor,
    SamplePathPasser,
    get_async_sample_paths,
)
from test_samples_helpers import get_sample_env_vars


class TestSamplesAsync(AzureRecordedTestCase):
    """Async test cases for samples."""

    # To run this test with a specific sample, use:
    # pytest tests/samples/test_samples_async.py::TestSamplesAsync::test_agent_tools_samples_async[sample_agent_memory_search_async]
    @servicePreparer()
    @pytest.mark.parametrize(
        "sample_path",
        get_async_sample_paths(
            "agents/tools",
            samples_to_skip=[
                "sample_agent_computer_use_async.py",
                "sample_agent_memory_search_async.py",  # Skipped until re-enabled and recorded on Foundry endpoint that supports the new versioning schema
            ],
        ),
    )
    @SamplePathPasser()
    @recorded_by_proxy_async(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    async def test_agent_tools_samples_async(self, sample_path: str, **kwargs) -> None:
        env_vars = get_sample_env_vars(kwargs)
        executor = AsyncSampleExecutor(
            self,
            sample_path,
            env_vars=env_vars,
            **kwargs,
        )
        await executor.execute_async()
        await executor.validate_print_calls_by_llm_async()

    @pytest.mark.parametrize(
        "sample_path",
        get_async_sample_paths(
            "memories",
            samples_to_skip=[
                "sample_memory_advanced_async.py",
                "sample_memory_basic_async.py",
                "sample_memory_crud_async.py",  # Skipped until re-enabled and recorded on Foundry endpoint that supports the new versioning schema
            ],
        ),
    )
    @servicePreparer()
    @SamplePathPasser()
    @recorded_by_proxy_async(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    # To run this test: pytest tests/samples/test_samples_async.py::TestSamplesAsync::test_memory_samples -s
    async def test_memory_samples(self, sample_path: str, **kwargs) -> None:
        env_vars = get_sample_env_vars(kwargs)
        executor = AsyncSampleExecutor(self, sample_path, env_vars=env_vars, **kwargs)
        await executor.execute_async()
        await executor.validate_print_calls_by_llm_async()

    @pytest.mark.parametrize(
        "sample_path",
        get_async_sample_paths(
            "agents",
            samples_to_skip=["sample_workflow_multi_agent_async.py"],
        ),
    )
    @servicePreparer()
    @SamplePathPasser()
    @recorded_by_proxy_async(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    async def test_agents_samples(self, sample_path: str, **kwargs) -> None:
        env_vars = get_sample_env_vars(kwargs)
        executor = AsyncSampleExecutor(self, sample_path, env_vars=env_vars, **kwargs)
        await executor.execute_async()
        await executor.validate_print_calls_by_llm_async()

    @pytest.mark.parametrize(
        "sample_path",
        get_async_sample_paths(
            "connections",
            samples_to_skip=[],
        ),
    )
    @servicePreparer()
    @SamplePathPasser()
    @recorded_by_proxy_async(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    async def test_connections_samples(self, sample_path: str, **kwargs) -> None:
        kwargs = kwargs.copy()
        kwargs["connection_name"] = "mcp"
        env_vars = get_sample_env_vars(kwargs)
        executor = AsyncSampleExecutor(self, sample_path, env_vars=env_vars, **kwargs)
        await executor.execute_async()
        await executor.validate_print_calls_by_llm_async()

    @pytest.mark.parametrize(
        "sample_path",
        get_async_sample_paths(
            "files",
            samples_to_skip=[
                "sample_files_async.py",  # Skipped until re-enabled and recorded on Foundry endpoint that supports the new versioning schema
            ],
        ),
    )
    @servicePreparer()
    @SamplePathPasser()
    @recorded_by_proxy_async(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    async def test_files_samples(self, sample_path: str, **kwargs) -> None:
        env_vars = get_sample_env_vars(kwargs)
        executor = AsyncSampleExecutor(self, sample_path, env_vars=env_vars, **kwargs)
        await executor.execute_async()
        await executor.validate_print_calls_by_llm_async()

    @pytest.mark.parametrize(
        "sample_path",
        get_async_sample_paths(
            "deployments",
            samples_to_skip=[],
        ),
    )
    @servicePreparer()
    @SamplePathPasser()
    @recorded_by_proxy_async(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    async def test_deployments_samples(self, sample_path: str, **kwargs) -> None:
        env_vars = get_sample_env_vars(kwargs)
        executor = AsyncSampleExecutor(self, sample_path, env_vars=env_vars, **kwargs)
        await executor.execute_async()
        await executor.validate_print_calls_by_llm_async()

    @pytest.mark.parametrize(
        "sample_path",
        get_async_sample_paths(
            "datasets",
            samples_to_skip=[
                "sample_datasets_async.py",  # Skipped until re-enabled and recorded on Foundry endpoint that supports the new versioning schema
            ],
        ),
    )
    @servicePreparer()
    @SamplePathPasser()
    @recorded_by_proxy_async(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    async def test_datasets_samples(self, sample_path: str, **kwargs) -> None:
        env_vars = get_sample_env_vars(kwargs)
        executor = AsyncSampleExecutor(self, sample_path, env_vars=env_vars, **kwargs)
        await executor.execute_async()
        if self.is_live:
            # Don't replay LLM validation since there probably a defect in proxy server fail to replay
            # Proxy server probably not able to parse the captured print content
            await executor.validate_print_calls_by_llm_async()

    @pytest.mark.parametrize(
        "sample_path",
        get_async_sample_paths(
            "chat_completions",
            samples_to_skip=[],
        ),
    )
    @servicePreparer()
    @SamplePathPasser()
    @recorded_by_proxy_async(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    async def test_chat_completions_samples(self, sample_path: str, **kwargs) -> None:
        env_vars = get_sample_env_vars(kwargs)
        executor = AsyncSampleExecutor(self, sample_path, env_vars=env_vars, **kwargs)
        await executor.execute_async()
        await executor.validate_print_calls_by_llm_async()

    @pytest.mark.parametrize(
        "sample_path",
        get_async_sample_paths(
            "hosted_agents",
            samples_to_skip=[],
        ),
    )
    @servicePreparer()
    @SamplePathPasser()
    @recorded_by_proxy_async(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    async def test_hosted_agents_samples(self, sample_path: str, **kwargs) -> None:
        env_vars = get_sample_env_vars(kwargs)
        executor = AsyncSampleExecutor(self, sample_path, env_vars=env_vars, **kwargs)
        await executor.execute_async()
        await executor.validate_print_calls_by_llm_async()



# coding=utf-8
from collections.abc import MutableMapping
import datetime
from typing import Any, Callable, Optional, TypeVar

from corehttp.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from corehttp.rest import HttpRequest, HttpResponse
from corehttp.runtime import PipelineClient
from corehttp.runtime.pipeline import PipelineResponse
from corehttp.utils import case_insensitive_dict

from ..._configuration import DurationClientConfiguration
from ..._utils.serialization import Deserializer, Serializer

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_query_default_request(*, input: datetime.timedelta, **kwargs: Any) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/encode/duration/query/default"

    # Construct parameters
    _params["input"] = _SERIALIZER.query("input", input, "duration")

    return HttpRequest(method="GET", url=_url, params=_params, **kwargs)


def build_query_iso8601_request(*, input: datetime.timedelta, **kwargs: Any) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/encode/duration/query/iso8601"

    # Construct parameters
    _params["input"] = _SERIALIZER.query("input", input, "duration")

    return HttpRequest(method="GET", url=_url, params=_params, **kwargs)


def build_query_int32_seconds_request(*, input: int, **kwargs: Any) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/encode/duration/query/int32-seconds"

    # Construct parameters
    _params["input"] = _SERIALIZER.query("input", input, "int")

    return HttpRequest(method="GET", url=_url, params=_params, **kwargs)


def build_query_int32_seconds_larger_unit_request(  # pylint: disable=name-too-long
    *, input: int, **kwargs: Any
) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/encode/duration/query/int32-seconds-larger-unit"

    # Construct parameters
    _params["input"] = _SERIALIZER.query("input", input, "int")

    return HttpRequest(method="GET", url=_url, params=_params, **kwargs)


def build_query_float_seconds_request(*, input: float, **kwargs: Any) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/encode/duration/query/float-seconds"

    # Construct parameters
    _params["input"] = _SERIALIZER.query("input", input, "float")

    return HttpRequest(method="GET", url=_url, params=_params, **kwargs)


def build_query_float_seconds_larger_unit_request(  # pylint: disable=name-too-long
    *, input: float, **kwargs: Any
) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/encode/duration/query/float-seconds-larger-unit"

    # Construct parameters
    _params["input"] = _SERIALIZER.query("input", input, "float")

    return HttpRequest(method="GET", url=_url, params=_params, **kwargs)


def build_query_float64_seconds_request(*, input: float, **kwargs: Any) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/encode/duration/query/float64-seconds"

    # Construct parameters
    _params["input"] = _SERIALIZER.query("input", input, "float")

    return HttpRequest(method="GET", url=_url, params=_params, **kwargs)


def build_query_int32_milliseconds_request(*, input: int, **kwargs: Any) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/encode/duration/query/int32-milliseconds"

    # Construct parameters
    _params["input"] = _SERIALIZER.query("input", input, "int")

    return HttpRequest(method="GET", url=_url, params=_params, **kwargs)


def build_query_int32_milliseconds_larger_unit_request(  # pylint: disable=name-too-long
    *, input: int, **kwargs: Any
) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/encode/duration/query/int32-milliseconds-larger-unit"

    # Construct parameters
    _params["input"] = _SERIALIZER.query("input", input, "int")

    return HttpRequest(method="GET", url=_url, params=_params, **kwargs)


def build_query_float_milliseconds_request(*, input: float, **kwargs: Any) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/encode/duration/query/float-milliseconds"

    # Construct parameters
    _params["input"] = _SERIALIZER.query("input", input, "float")

    return HttpRequest(method="GET", url=_url, params=_params, **kwargs)


def build_query_float_milliseconds_larger_unit_request(  # pylint: disable=name-too-long
    *, input: float, **kwargs: Any
) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/encode/duration/query/float-milliseconds-larger-unit"

    # Construct parameters
    _params["input"] = _SERIALIZER.query("input", input, "float")

    return HttpRequest(method="GET", url=_url, params=_params, **kwargs)


def build_query_float64_milliseconds_request(*, input: float, **kwargs: Any) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/encode/duration/query/float64-milliseconds"

    # Construct parameters
    _params["input"] = _SERIALIZER.query("input", input, "float")

    return HttpRequest(method="GET", url=_url, params=_params, **kwargs)


def build_query_int32_seconds_array_request(*, input: list[int], **kwargs: Any) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/encode/duration/query/int32-seconds-array"

    # Construct parameters
    _params["input"] = _SERIALIZER.query("input", input, "[int]", div=",")

    return HttpRequest(method="GET", url=_url, params=_params, **kwargs)


def build_query_int32_milliseconds_array_request(  # pylint: disable=name-too-long
    *, input: list[int], **kwargs: Any
) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/encode/duration/query/int32-milliseconds-array"

    # Construct parameters
    _params["input"] = _SERIALIZER.query("input", input, "[int]", div=",")

    return HttpRequest(method="GET", url=_url, params=_params, **kwargs)


class QueryOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~encode.duration.DurationClient`'s
        :attr:`query` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: PipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: DurationClientConfiguration = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    def default(  # pylint: disable=inconsistent-return-statements
        self, *, input: datetime.timedelta, **kwargs: Any
    ) -> None:
        """default.

        :keyword input: Required.
        :paramtype input: ~datetime.timedelta
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_query_default_request(
            input=input,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    def iso8601(  # pylint: disable=inconsistent-return-statements
        self, *, input: datetime.timedelta, **kwargs: Any
    ) -> None:
        """iso8601.

        :keyword input: Required.
        :paramtype input: ~datetime.timedelta
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_query_iso8601_request(
            input=input,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    def int32_seconds(self, *, input: int, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """int32_seconds.

        :keyword input: Required.
        :paramtype input: int
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_query_int32_seconds_request(
            input=input,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    def int32_seconds_larger_unit(  # pylint: disable=inconsistent-return-statements
        self, *, input: int, **kwargs: Any
    ) -> None:
        """int32_seconds_larger_unit.

        :keyword input: Required.
        :paramtype input: int
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_query_int32_seconds_larger_unit_request(
            input=input,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    def float_seconds(self, *, input: float, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """float_seconds.

        :keyword input: Required.
        :paramtype input: float
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_query_float_seconds_request(
            input=input,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    def float_seconds_larger_unit(  # pylint: disable=inconsistent-return-statements
        self, *, input: float, **kwargs: Any
    ) -> None:
        """float_seconds_larger_unit.

        :keyword input: Required.
        :paramtype input: float
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_query_float_seconds_larger_unit_request(
            input=input,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    def float64_seconds(self, *, input: float, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """float64_seconds.

        :keyword input: Required.
        :paramtype input: float
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_query_float64_seconds_request(
            input=input,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    def int32_milliseconds(  # pylint: disable=inconsistent-return-statements
        self, *, input: int, **kwargs: Any
    ) -> None:
        """int32_milliseconds.

        :keyword input: Required.
        :paramtype input: int
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_query_int32_milliseconds_request(
            input=input,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    def int32_milliseconds_larger_unit(  # pylint: disable=inconsistent-return-statements
        self, *, input: int, **kwargs: Any
    ) -> None:
        """int32_milliseconds_larger_unit.

        :keyword input: Required.
        :paramtype input: int
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_query_int32_milliseconds_larger_unit_request(
            input=input,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    def float_milliseconds(  # pylint: disable=inconsistent-return-statements
        self, *, input: float, **kwargs: Any
    ) -> None:
        """float_milliseconds.

        :keyword input: Required.
        :paramtype input: float
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_query_float_milliseconds_request(
            input=input,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    def float_milliseconds_larger_unit(  # pylint: disable=inconsistent-return-statements
        self, *, input: float, **kwargs: Any
    ) -> None:
        """float_milliseconds_larger_unit.

        :keyword input: Required.
        :paramtype input: float
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_query_float_milliseconds_larger_unit_request(
            input=input,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    def float64_milliseconds(  # pylint: disable=inconsistent-return-statements
        self, *, input: float, **kwargs: Any
    ) -> None:
        """float64_milliseconds.

        :keyword input: Required.
        :paramtype input: float
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_query_float64_milliseconds_request(
            input=input,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    def int32_seconds_array(  # pylint: disable=inconsistent-return-statements
        self, *, input: list[int], **kwargs: Any
    ) -> None:
        """int32_seconds_array.

        :keyword input: Required.
        :paramtype input: list[int]
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_query_int32_seconds_array_request(
            input=input,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    def int32_milliseconds_array(  # pylint: disable=inconsistent-return-statements
        self, *, input: list[int], **kwargs: Any
    ) -> None:
        """int32_milliseconds_array.

        :keyword input: Required.
        :paramtype input: list[int]
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_query_int32_milliseconds_array_request(
            input=input,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.appcontainers import ContainerAppsAPIClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-appcontainers
# USAGE
    python container_apps_kind_function_app_create_or_update.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = ContainerAppsAPIClient(
        credential=DefaultAzureCredential(),
        subscription_id="00000000-0000-0000-0000-000000000000",
    )

    response = client.container_apps.begin_create_or_update(
        resource_group_name="rg",
        container_app_name="testcontainerAppFunctionKind",
        container_app_envelope={
            "kind": "functionapp",
            "location": "East Us",
            "managedBy": "/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/rg/providers/Microsoft.Web/sites/testcontainerAppFunctionKind",
            "properties": {
                "configuration": {
                    "activeRevisionsMode": "Single",
                    "ingress": {"allowInsecure": False, "external": True, "targetPort": 80},
                },
                "managedEnvironmentId": "/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/rg/providers/Microsoft.App/managedEnvironments/testmanagedenv3",
                "template": {
                    "containers": [
                        {
                            "env": [
                                {
                                    "name": "AzureWebJobsStorage",
                                    "value": "DefaultEndpointsProtocol=https;AccountName=mystorageaccount;AccountKey=mykey;EndpointSuffix=core.windows.net",
                                },
                                {"name": "FUNCTIONS_WORKER_RUNTIME", "value": "dotnet"},
                                {"name": "WEBSITES_ENABLE_APP_SERVICE_STORAGE", "value": "false"},
                            ],
                            "image": "mcr.microsoft.com/azure-functions/dotnet:4",
                            "name": "function-app-container",
                            "resources": {"cpu": 0.5, "memory": "1.0Gi"},
                        }
                    ],
                    "scale": {"cooldownPeriod": 300, "maxReplicas": 10, "minReplicas": 0, "pollingInterval": 30},
                },
            },
        },
    ).result()
    print(response)


# x-ms-original-file: specification/app/resource-manager/Microsoft.App/ContainerApps/stable/2025-07-01/examples/ContainerApps_Kind_FunctionApp_CreateOrUpdate.json
if __name__ == "__main__":
    main()



# coding=utf-8

from copy import deepcopy
from typing import Any, Awaitable
from typing_extensions import Self

from corehttp.rest import AsyncHttpResponse, HttpRequest
from corehttp.runtime import AsyncPipelineClient, policies

from .._utils.serialization import Deserializer, Serializer
from ._configuration import XmlClientConfiguration
from .operations import (
    ModelWithArrayOfModelValueOperations,
    ModelWithAttributesValueOperations,
    ModelWithDatetimeValueOperations,
    ModelWithDictionaryValueOperations,
    ModelWithEmptyArrayValueOperations,
    ModelWithEncodedNamesValueOperations,
    ModelWithEnumValueOperations,
    ModelWithNamespaceOnPropertiesValueOperations,
    ModelWithNamespaceValueOperations,
    ModelWithNestedModelValueOperations,
    ModelWithOptionalFieldValueOperations,
    ModelWithRenamedArraysValueOperations,
    ModelWithRenamedAttributeValueOperations,
    ModelWithRenamedFieldsValueOperations,
    ModelWithRenamedNestedModelValueOperations,
    ModelWithRenamedPropertyValueOperations,
    ModelWithRenamedUnwrappedModelArrayValueOperations,
    ModelWithRenamedWrappedAndItemModelArrayValueOperations,
    ModelWithRenamedWrappedModelArrayValueOperations,
    ModelWithSimpleArraysValueOperations,
    ModelWithTextValueOperations,
    ModelWithUnwrappedArrayValueOperations,
    ModelWithUnwrappedModelArrayValueOperations,
    ModelWithWrappedPrimitiveCustomItemNamesValueOperations,
    SimpleModelValueOperations,
    XmlErrorValueOperations,
)


class XmlClient:  # pylint: disable=client-accepts-api-version-keyword,too-many-instance-attributes
    """Sends and receives bodies in XML format.

    :ivar simple_model_value: SimpleModelValueOperations operations
    :vartype simple_model_value: payload.xml.aio.operations.SimpleModelValueOperations
    :ivar model_with_renamed_property_value: ModelWithRenamedPropertyValueOperations operations
    :vartype model_with_renamed_property_value:
     payload.xml.aio.operations.ModelWithRenamedPropertyValueOperations
    :ivar model_with_renamed_fields_value: ModelWithRenamedFieldsValueOperations operations
    :vartype model_with_renamed_fields_value:
     payload.xml.aio.operations.ModelWithRenamedFieldsValueOperations
    :ivar model_with_nested_model_value: ModelWithNestedModelValueOperations operations
    :vartype model_with_nested_model_value:
     payload.xml.aio.operations.ModelWithNestedModelValueOperations
    :ivar model_with_renamed_nested_model_value: ModelWithRenamedNestedModelValueOperations
     operations
    :vartype model_with_renamed_nested_model_value:
     payload.xml.aio.operations.ModelWithRenamedNestedModelValueOperations
    :ivar model_with_simple_arrays_value: ModelWithSimpleArraysValueOperations operations
    :vartype model_with_simple_arrays_value:
     payload.xml.aio.operations.ModelWithSimpleArraysValueOperations
    :ivar model_with_unwrapped_array_value: ModelWithUnwrappedArrayValueOperations operations
    :vartype model_with_unwrapped_array_value:
     payload.xml.aio.operations.ModelWithUnwrappedArrayValueOperations
    :ivar model_with_renamed_arrays_value: ModelWithRenamedArraysValueOperations operations
    :vartype model_with_renamed_arrays_value:
     payload.xml.aio.operations.ModelWithRenamedArraysValueOperations
    :ivar model_with_wrapped_primitive_custom_item_names_value:
     ModelWithWrappedPrimitiveCustomItemNamesValueOperations operations
    :vartype model_with_wrapped_primitive_custom_item_names_value:
     payload.xml.aio.operations.ModelWithWrappedPrimitiveCustomItemNamesValueOperations
    :ivar model_with_array_of_model_value: ModelWithArrayOfModelValueOperations operations
    :vartype model_with_array_of_model_value:
     payload.xml.aio.operations.ModelWithArrayOfModelValueOperations
    :ivar model_with_unwrapped_model_array_value: ModelWithUnwrappedModelArrayValueOperations
     operations
    :vartype model_with_unwrapped_model_array_value:
     payload.xml.aio.operations.ModelWithUnwrappedModelArrayValueOperations
    :ivar model_with_renamed_wrapped_model_array_value:
     ModelWithRenamedWrappedModelArrayValueOperations operations
    :vartype model_with_renamed_wrapped_model_array_value:
     payload.xml.aio.operations.ModelWithRenamedWrappedModelArrayValueOperations
    :ivar model_with_renamed_unwrapped_model_array_value:
     ModelWithRenamedUnwrappedModelArrayValueOperations operations
    :vartype model_with_renamed_unwrapped_model_array_value:
     payload.xml.aio.operations.ModelWithRenamedUnwrappedModelArrayValueOperations
    :ivar model_with_renamed_wrapped_and_item_model_array_value:
     ModelWithRenamedWrappedAndItemModelArrayValueOperations operations
    :vartype model_with_renamed_wrapped_and_item_model_array_value:
     payload.xml.aio.operations.ModelWithRenamedWrappedAndItemModelArrayValueOperations
    :ivar model_with_attributes_value: ModelWithAttributesValueOperations operations
    :vartype model_with_attributes_value:
     payload.xml.aio.operations.ModelWithAttributesValueOperations
    :ivar model_with_renamed_attribute_value: ModelWithRenamedAttributeValueOperations operations
    :vartype model_with_renamed_attribute_value:
     payload.xml.aio.operations.ModelWithRenamedAttributeValueOperations
    :ivar model_with_namespace_value: ModelWithNamespaceValueOperations operations
    :vartype model_with_namespace_value:
     payload.xml.aio.operations.ModelWithNamespaceValueOperations
    :ivar model_with_namespace_on_properties_value: ModelWithNamespaceOnPropertiesValueOperations
     operations
    :vartype model_with_namespace_on_properties_value:
     payload.xml.aio.operations.ModelWithNamespaceOnPropertiesValueOperations
    :ivar model_with_text_value: ModelWithTextValueOperations operations
    :vartype model_with_text_value: payload.xml.aio.operations.ModelWithTextValueOperations
    :ivar model_with_optional_field_value: ModelWithOptionalFieldValueOperations operations
    :vartype model_with_optional_field_value:
     payload.xml.aio.operations.ModelWithOptionalFieldValueOperations
    :ivar model_with_empty_array_value: ModelWithEmptyArrayValueOperations operations
    :vartype model_with_empty_array_value:
     payload.xml.aio.operations.ModelWithEmptyArrayValueOperations
    :ivar model_with_dictionary_value: ModelWithDictionaryValueOperations operations
    :vartype model_with_dictionary_value:
     payload.xml.aio.operations.ModelWithDictionaryValueOperations
    :ivar model_with_encoded_names_value: ModelWithEncodedNamesValueOperations operations
    :vartype model_with_encoded_names_value:
     payload.xml.aio.operations.ModelWithEncodedNamesValueOperations
    :ivar model_with_enum_value: ModelWithEnumValueOperations operations
    :vartype model_with_enum_value: payload.xml.aio.operations.ModelWithEnumValueOperations
    :ivar model_with_datetime_value: ModelWithDatetimeValueOperations operations
    :vartype model_with_datetime_value: payload.xml.aio.operations.ModelWithDatetimeValueOperations
    :ivar xml_error_value: XmlErrorValueOperations operations
    :vartype xml_error_value: payload.xml.aio.operations.XmlErrorValueOperations
    :keyword endpoint: Service host. Default value is "http://localhost:3000".
    :paramtype endpoint: str
    """

    def __init__(  # pylint: disable=missing-client-constructor-parameter-credential
        self, *, endpoint: str = "http://localhost:3000", **kwargs: Any
    ) -> None:
        _endpoint = "{endpoint}"
        self._config = XmlClientConfiguration(endpoint=endpoint, **kwargs)

        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.logging_policy,
            ]
        self._client: AsyncPipelineClient = AsyncPipelineClient(endpoint=_endpoint, policies=_policies, **kwargs)

        self._serialize = Serializer()
        self._deserialize = Deserializer()
        self._serialize.client_side_validation = False
        self.simple_model_value = SimpleModelValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_renamed_property_value = ModelWithRenamedPropertyValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_renamed_fields_value = ModelWithRenamedFieldsValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_nested_model_value = ModelWithNestedModelValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_renamed_nested_model_value = ModelWithRenamedNestedModelValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_simple_arrays_value = ModelWithSimpleArraysValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_unwrapped_array_value = ModelWithUnwrappedArrayValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_renamed_arrays_value = ModelWithRenamedArraysValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_wrapped_primitive_custom_item_names_value = (
            ModelWithWrappedPrimitiveCustomItemNamesValueOperations(
                self._client, self._config, self._serialize, self._deserialize
            )
        )
        self.model_with_array_of_model_value = ModelWithArrayOfModelValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_unwrapped_model_array_value = ModelWithUnwrappedModelArrayValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_renamed_wrapped_model_array_value = ModelWithRenamedWrappedModelArrayValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_renamed_unwrapped_model_array_value = ModelWithRenamedUnwrappedModelArrayValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_renamed_wrapped_and_item_model_array_value = (
            ModelWithRenamedWrappedAndItemModelArrayValueOperations(
                self._client, self._config, self._serialize, self._deserialize
            )
        )
        self.model_with_attributes_value = ModelWithAttributesValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_renamed_attribute_value = ModelWithRenamedAttributeValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_namespace_value = ModelWithNamespaceValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_namespace_on_properties_value = ModelWithNamespaceOnPropertiesValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_text_value = ModelWithTextValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_optional_field_value = ModelWithOptionalFieldValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_empty_array_value = ModelWithEmptyArrayValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_dictionary_value = ModelWithDictionaryValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_encoded_names_value = ModelWithEncodedNamesValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_enum_value = ModelWithEnumValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.model_with_datetime_value = ModelWithDatetimeValueOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.xml_error_value = XmlErrorValueOperations(self._client, self._config, self._serialize, self._deserialize)

    def send_request(
        self, request: HttpRequest, *, stream: bool = False, **kwargs: Any
    ) -> Awaitable[AsyncHttpResponse]:
        """Runs the network request through the client's chained policies.

        >>> from corehttp.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = await client.send_request(request)
        <AsyncHttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~corehttp.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~corehttp.rest.AsyncHttpResponse
        """

        request_copy = deepcopy(request)
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }

        request_copy.url = self._client.format_url(request_copy.url, **path_format_arguments)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    async def close(self) -> None:
        await self._client.close()

    async def __aenter__(self) -> Self:
        await self._client.__aenter__()
        return self

    async def __aexit__(self, *exc_details: Any) -> None:
        await self._client.__aexit__(*exc_details)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
"""Unit tests for startup configuration logging and URI masking."""
import logging

import pytest

from azure.ai.agentserver.core._base import _mask_uri, _NOT_SET


class TestMaskUri:
    """Tests for the _mask_uri helper that redacts sensitive URI components."""

    def test_empty_string_returns_not_set(self) -> None:
        assert _mask_uri("") == _NOT_SET

    def test_none_like_empty_returns_not_set(self) -> None:
        # Whitespace-only
        assert _mask_uri("   ") == _NOT_SET

    def test_https_uri_strips_path_and_query(self) -> None:
        result = _mask_uri("https://myproject.azure.com/subscriptions/abc?api-version=2024")
        assert result == "https://myproject.azure.com"

    def test_http_uri_with_port(self) -> None:
        result = _mask_uri("http://localhost:8080/some/path")
        assert result == "http://localhost:8080"

    def test_https_uri_with_credentials_in_userinfo(self) -> None:
        result = _mask_uri("https://user:password@myhost.com/resource")
        assert result == "https://myhost.com"

    def test_uri_with_only_scheme_and_host(self) -> None:
        result = _mask_uri("https://myhost.com")
        assert result == "https://myhost.com"

    def test_non_uri_string_returns_redacted(self) -> None:
        # A bare string with no scheme won't parse properly
        result = _mask_uri("not-a-uri")
        assert result == "(redacted)"

    def test_connection_string_format_is_redacted(self) -> None:
        # Connection strings like AppInsights don't have scheme://host format
        result = _mask_uri("InstrumentationKey=abc;IngestionEndpoint=https://x.in.ai")
        assert result == "(redacted)"


class TestStartupConfigurationLogging:
    """Tests that startup configuration is logged during lifespan."""

    @pytest.fixture
    def _clean_env(self, monkeypatch: pytest.MonkeyPatch) -> None:
        """Remove all platform env vars to get a clean baseline."""
        for var in [
            "FOUNDRY_AGENT_NAME",
            "FOUNDRY_AGENT_VERSION",
            "FOUNDRY_HOSTING_ENVIRONMENT",
            "FOUNDRY_PROJECT_ENDPOINT",
            "FOUNDRY_PROJECT_ARM_ID",
            "FOUNDRY_AGENT_SESSION_ID",
            "PORT",
            "APPLICATIONINSIGHTS_CONNECTION_STRING",
            "OTEL_EXPORTER_OTLP_ENDPOINT",
            "SSE_KEEPALIVE_INTERVAL",
        ]:
            monkeypatch.delenv(var, raising=False)

    @pytest.mark.usefixtures("_clean_env")
    @pytest.mark.asyncio
    async def test_startup_logs_platform_environment(self, caplog: pytest.LogCaptureFixture) -> None:
        """Lifespan startup emits platform environment log line."""
        from azure.ai.agentserver.core import AgentServerHost

        app = AgentServerHost()

        with caplog.at_level(logging.INFO, logger="azure.ai.agentserver"):
            async with app.router.lifespan_context(app):
                pass

        platform_logs = [r for r in caplog.records if "Platform environment" in r.message]
        assert len(platform_logs) == 1
        msg = platform_logs[0].message
        assert "is_hosted=False" in msg
        assert "port=8088" in msg

    @pytest.mark.usefixtures("_clean_env")
    @pytest.mark.asyncio
    async def test_startup_logs_connectivity(self, caplog: pytest.LogCaptureFixture) -> None:
        """Lifespan startup emits connectivity log line with masked URIs."""
        from azure.ai.agentserver.core import AgentServerHost

        app = AgentServerHost()

        with caplog.at_level(logging.INFO, logger="azure.ai.agentserver"):
            async with app.router.lifespan_context(app):
                pass

        connectivity_logs = [r for r in caplog.records if "Connectivity" in r.message]
        assert len(connectivity_logs) == 1
        msg = connectivity_logs[0].message
        assert "project_endpoint=(not set)" in msg
        assert "appinsights_configured=False" in msg

    @pytest.mark.usefixtures("_clean_env")
    @pytest.mark.asyncio
    async def test_startup_logs_host_options(self, caplog: pytest.LogCaptureFixture) -> None:
        """Lifespan startup emits host options log line."""
        from azure.ai.agentserver.core import AgentServerHost

        app = AgentServerHost()

        with caplog.at_level(logging.INFO, logger="azure.ai.agentserver"):
            async with app.router.lifespan_context(app):
                pass

        host_logs = [r for r in caplog.records if "Host options" in r.message]
        assert len(host_logs) == 1
        msg = host_logs[0].message
        assert "shutdown_timeout=30s" in msg
        assert "protocols=" in msg

    @pytest.mark.asyncio
    async def test_startup_masks_project_endpoint(
        self, monkeypatch: pytest.MonkeyPatch, caplog: pytest.LogCaptureFixture
    ) -> None:
        """Project endpoint URI is masked to scheme://host only."""
        monkeypatch.setenv("FOUNDRY_PROJECT_ENDPOINT", "https://myproject.azure.com/sub/123?key=secret")
        monkeypatch.delenv("FOUNDRY_HOSTING_ENVIRONMENT", raising=False)
        monkeypatch.delenv("PORT", raising=False)

        from azure.ai.agentserver.core import AgentServerHost

        app = AgentServerHost()

        with caplog.at_level(logging.INFO, logger="azure.ai.agentserver"):
            async with app.router.lifespan_context(app):
                pass

        connectivity_logs = [r for r in caplog.records if "Connectivity" in r.message]
        assert len(connectivity_logs) == 1
        msg = connectivity_logs[0].message
        assert "project_endpoint=https://myproject.azure.com" in msg
        # Must NOT contain the path or query
        assert "/sub/123" not in msg
        assert "key=secret" not in msg

    @pytest.mark.asyncio
    async def test_appinsights_connection_string_never_logged(
        self, monkeypatch: pytest.MonkeyPatch, caplog: pytest.LogCaptureFixture
    ) -> None:
        """AppInsights connection string value must never appear in logs."""
        secret = "InstrumentationKey=00000000-0000-0000-0000-000000000000;IngestionEndpoint=https://dc.ai"
        monkeypatch.setenv("APPLICATIONINSIGHTS_CONNECTION_STRING", secret)
        monkeypatch.delenv("FOUNDRY_HOSTING_ENVIRONMENT", raising=False)
        monkeypatch.delenv("PORT", raising=False)

        from azure.ai.agentserver.core import AgentServerHost

        app = AgentServerHost(configure_observability=None)

        with caplog.at_level(logging.DEBUG, logger="azure.ai.agentserver"):
            async with app.router.lifespan_context(app):
                pass

        # The boolean flag should be logged
        connectivity_logs = [r for r in caplog.records if "Connectivity" in r.message]
        assert len(connectivity_logs) == 1
        assert "appinsights_configured=True" in connectivity_logs[0].message

        # The actual connection string must NEVER appear in any log record
        all_messages = " ".join(r.message for r in caplog.records)
        assert "InstrumentationKey" not in all_messages
        assert secret not in all_messages



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
"""Tests for the trace_function decorator with asynchronous functions."""

# pylint: disable=unused-argument

import pytest
from opentelemetry import trace
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import SimpleSpanProcessor
from gen_ai_trace_verifier import GenAiTraceVerifier  # pylint: disable=import-error
from memory_trace_exporter import MemoryTraceExporter  # pylint: disable=import-error
from azure.ai.projects.telemetry._trace_function import trace_function


class TestTraceFunctionDecoratorAsync:
    """Tests for trace_function decorator with asynchronous functions."""

    @pytest.fixture(scope="function")
    def setup_telemetry(self):  # pylint: disable=attribute-defined-outside-init
        """Setup telemetry for tests."""
        tracer_provider = TracerProvider()
        trace._TRACER_PROVIDER = tracer_provider
        self.exporter = MemoryTraceExporter()  # pylint: disable=attribute-defined-outside-init
        span_processor = SimpleSpanProcessor(self.exporter)
        tracer_provider.add_span_processor(span_processor)
        yield
        self.exporter.shutdown()
        trace._TRACER_PROVIDER = None

    @pytest.mark.asyncio
    async def test_async_basic_function_with_primitives(self, setup_telemetry):
        """Test decorator with an async function that has primitive type parameters."""

        @trace_function()
        async def add_numbers_async(a: int, b: int) -> int:
            return a + b

        result = await add_numbers_async(5, 3)
        assert result == 8

        spans = self.exporter.get_spans_by_name("add_numbers_async")
        assert len(spans) == 1
        span = spans[0]

        expected_attributes = [
            ("code.function.parameter.a", 5),
            ("code.function.parameter.b", 3),
            ("code.function.return.value", 8),
        ]
        attributes_match = GenAiTraceVerifier().check_decorator_span_attributes(span, expected_attributes)
        assert attributes_match is True

    @pytest.mark.asyncio
    async def test_async_function_with_string_parameters(self, setup_telemetry):
        """Test decorator with async function and string parameters."""

        @trace_function()
        async def greet_async(name: str, greeting: str = "Hello") -> str:
            return f"{greeting}, {name}!"

        result = await greet_async("Bob", "Good morning")
        assert result == "Good morning, Bob!"

        spans = self.exporter.get_spans_by_name("greet_async")
        assert len(spans) == 1
        span = spans[0]

        expected_attributes = [
            ("code.function.parameter.name", "Bob"),
            ("code.function.parameter.greeting", "Good morning"),
            ("code.function.return.value", "Good morning, Bob!"),
        ]
        attributes_match = GenAiTraceVerifier().check_decorator_span_attributes(span, expected_attributes)
        assert attributes_match is True

    @pytest.mark.asyncio
    async def test_async_function_with_list_parameter(self, setup_telemetry):
        """Test decorator with async function and list parameters."""

        @trace_function()
        async def sum_list_async(numbers: list) -> int:
            return sum(numbers)

        result = await sum_list_async([10, 20, 30, 40])
        assert result == 100

        spans = self.exporter.get_spans_by_name("sum_list_async")
        assert len(spans) == 1
        span = spans[0]

        expected_attributes = [
            ("code.function.parameter.numbers", [10, 20, 30, 40]),
            ("code.function.return.value", 100),
        ]
        attributes_match = GenAiTraceVerifier().check_decorator_span_attributes(span, expected_attributes)
        assert attributes_match is True

    @pytest.mark.asyncio
    async def test_async_function_with_dict_parameter(self, setup_telemetry):
        """Test decorator with async function and dict parameters."""

        @trace_function()
        async def get_value_async(data: dict, key: str) -> str:
            return data.get(key, "not found")

        result = await get_value_async({"city": "Seattle", "state": "WA"}, "city")
        assert result == "Seattle"

        spans = self.exporter.get_spans_by_name("get_value_async")
        assert len(spans) == 1
        span = spans[0]

        expected_dict_str = str({"city": "Seattle", "state": "WA"})
        expected_attributes = [
            ("code.function.parameter.data", expected_dict_str),
            ("code.function.parameter.key", "city"),
            ("code.function.return.value", "Seattle"),
        ]
        attributes_match = GenAiTraceVerifier().check_decorator_span_attributes(span, expected_attributes)
        assert attributes_match is True

    @pytest.mark.asyncio
    async def test_async_function_with_nested_collections(self, setup_telemetry):
        """Test decorator with async function and nested collections."""

        @trace_function()
        async def process_nested_async(data: list) -> int:
            total = 0
            for sublist in data:
                total += sum(sublist)
            return total

        nested_data = [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
        result = await process_nested_async(nested_data)
        assert result == 45

        spans = self.exporter.get_spans_by_name("process_nested_async")
        assert len(spans) == 1
        span = spans[0]

        expected_data_str = str(nested_data)
        expected_attributes = [
            ("code.function.parameter.data", expected_data_str),
            ("code.function.return.value", 45),
        ]
        attributes_match = GenAiTraceVerifier().check_decorator_span_attributes(span, expected_attributes)
        assert attributes_match is True

    @pytest.mark.asyncio
    async def test_async_function_with_custom_span_name(self, setup_telemetry):
        """Test decorator with custom span name on async function."""

        @trace_function(span_name="async_custom_operation")
        async def calculate_async(x: int, y: int) -> int:
            return x * y

        result = await calculate_async(6, 9)
        assert result == 54

        spans = self.exporter.get_spans_by_name("async_custom_operation")
        assert len(spans) == 1
        span = spans[0]

        expected_attributes = [
            ("code.function.parameter.x", 6),
            ("code.function.parameter.y", 9),
            ("code.function.return.value", 54),
        ]
        attributes_match = GenAiTraceVerifier().check_decorator_span_attributes(span, expected_attributes)
        assert attributes_match is True

    @pytest.mark.asyncio
    async def test_async_function_with_no_return_value(self, setup_telemetry):
        """Test decorator with async function that returns None."""

        @trace_function()
        async def log_message_async(message: str) -> None:
            pass  # Just a placeholder

        result = await log_message_async("Async test message")
        assert result is None

        spans = self.exporter.get_spans_by_name("log_message_async")
        assert len(spans) == 1
        span = spans[0]

        expected_attributes = [
            ("code.function.parameter.message", "Async test message"),
        ]
        attributes_match = GenAiTraceVerifier().check_decorator_span_attributes(span, expected_attributes)
        assert attributes_match is True

    @pytest.mark.asyncio
    async def test_async_function_with_boolean_parameters(self, setup_telemetry):
        """Test decorator with async function and boolean parameters."""

        @trace_function()
        async def check_status_async(is_active: bool, is_verified: bool) -> str:
            if is_active and is_verified:
                return "approved"
            if is_active:
                return "pending"
            return "inactive"

        result = await check_status_async(True, True)
        assert result == "approved"

        spans = self.exporter.get_spans_by_name("check_status_async")
        assert len(spans) == 1
        span = spans[0]

        expected_attributes = [
            ("code.function.parameter.is_active", True),
            ("code.function.parameter.is_verified", True),
            ("code.function.return.value", "approved"),
        ]
        attributes_match = GenAiTraceVerifier().check_decorator_span_attributes(span, expected_attributes)
        assert attributes_match is True

    @pytest.mark.asyncio
    async def test_async_function_with_float_parameters(self, setup_telemetry):
        """Test decorator with async function and float parameters."""

        @trace_function()
        async def calculate_area_async(width: float, height: float) -> float:
            return width * height

        result = await calculate_area_async(12.5, 8.3)
        assert abs(result - 103.75) < 0.01

        spans = self.exporter.get_spans_by_name("calculate_area_async")
        assert len(spans) == 1
        span = spans[0]

        expected_attributes = [
            ("code.function.parameter.width", 12.5),
            ("code.function.parameter.height", 8.3),
            ("code.function.return.value", result),
        ]
        attributes_match = GenAiTraceVerifier().check_decorator_span_attributes(span, expected_attributes)
        assert attributes_match is True

    @pytest.mark.asyncio
    async def test_async_function_with_tuple_parameter(self, setup_telemetry):
        """Test decorator with async function and tuple parameters."""

        @trace_function()
        async def get_coordinates_async(point: tuple) -> str:
            return f"Position: ({point[0]}, {point[1]}, {point[2]})"

        result = await get_coordinates_async((10, 20, 30))
        assert result == "Position: (10, 20, 30)"

        spans = self.exporter.get_spans_by_name("get_coordinates_async")
        assert len(spans) == 1
        span = spans[0]

        expected_attributes = [
            ("code.function.parameter.point", (10, 20, 30)),
            ("code.function.return.value", "Position: (10, 20, 30)"),
        ]
        attributes_match = GenAiTraceVerifier().check_decorator_span_attributes(span, expected_attributes)
        assert attributes_match is True

    @pytest.mark.asyncio
    async def test_async_function_with_set_parameter(self, setup_telemetry):
        """Test decorator with async function and set parameters."""

        @trace_function()
        async def count_unique_async(items: set) -> int:
            return len(items)

        result = await count_unique_async({10, 20, 30, 40, 50})
        assert result == 5

        spans = self.exporter.get_spans_by_name("count_unique_async")
        assert len(spans) == 1
        span = spans[0]

        # Sets are converted to strings, but we need to compare the actual set values
        # since set string representation order is non-deterministic
        assert span.attributes is not None
        assert "code.function.parameter.items" in span.attributes
        assert "code.function.return.value" in span.attributes

        # Convert the string back to a set for comparison
        import ast

        items_str = span.attributes["code.function.parameter.items"]
        assert isinstance(items_str, str)
        items_value = ast.literal_eval(items_str)
        assert items_value == {10, 20, 30, 40, 50}
        assert span.attributes["code.function.return.value"] == 5

    @pytest.mark.asyncio
    async def test_async_function_with_exception(self, setup_telemetry):
        """Test decorator records exception information in async functions."""

        @trace_function()
        async def divide_async(a: int, b: int) -> float:
            return a / b

        with pytest.raises(ZeroDivisionError):
            await divide_async(100, 0)

        spans = self.exporter.get_spans_by_name("divide_async")
        assert len(spans) == 1
        span = spans[0]

        expected_attributes = [
            ("code.function.parameter.a", 100),
            ("code.function.parameter.b", 0),
            ("error.type", "ZeroDivisionError"),
        ]
        attributes_match = GenAiTraceVerifier().check_decorator_span_attributes(span, expected_attributes)
        assert attributes_match is True

        # Verify exception was recorded as an event
        assert len(span.events) > 0
        exception_event = None
        for event in span.events:
            if event.name == "exception":
                exception_event = event
                break
        assert exception_event is not None

    @pytest.mark.asyncio
    async def test_async_function_with_mixed_parameters(self, setup_telemetry):
        """Test decorator with async function and mixed parameter types."""

        @trace_function()
        async def process_data_async(name: str, count: int, active: bool, scores: list) -> dict:
            return {
                "name": name.upper(),
                "count": count * 2,
                "active": active,
                "total": sum(scores),
            }

        result = await process_data_async("async_test", 3, False, [100, 200, 300])
        assert result["name"] == "ASYNC_TEST"
        assert result["count"] == 6
        assert result["total"] == 600

        spans = self.exporter.get_spans_by_name("process_data_async")
        assert len(spans) == 1
        span = spans[0]

        expected_result_str = str(result)
        expected_attributes = [
            ("code.function.parameter.name", "async_test"),
            ("code.function.parameter.count", 3),
            ("code.function.parameter.active", False),
            ("code.function.parameter.scores", [100, 200, 300]),
            ("code.function.return.value", expected_result_str),
        ]
        attributes_match = GenAiTraceVerifier().check_decorator_span_attributes(span, expected_attributes)
        assert attributes_match is True

    @pytest.mark.asyncio
    async def test_async_function_with_default_parameters(self, setup_telemetry):  # pylint: disable=unused-argument
        """Test decorator with async function using default parameters."""

        @trace_function()
        async def create_user_async(name: str, role: str = "user", active: bool = True) -> dict:
            return {"name": name, "role": role, "active": active}

        result = await create_user_async("Charlie")
        assert result["name"] == "Charlie"
        assert result["role"] == "user"
        assert result["active"] is True

        spans = self.exporter.get_spans_by_name("create_user_async")
        assert len(spans) == 1
        span = spans[0]

        expected_result_str = str(result)
        expected_attributes = [
            ("code.function.parameter.name", "Charlie"),
            ("code.function.parameter.role", "user"),
            ("code.function.parameter.active", True),
            ("code.function.return.value", expected_result_str),
        ]
        attributes_match = GenAiTraceVerifier().check_decorator_span_attributes(span, expected_attributes)
        assert attributes_match is True

    @pytest.mark.asyncio
    async def test_async_function_list_return_value(self, setup_telemetry):  # pylint: disable=unused-argument
        """Test decorator with async function returning a list."""

        @trace_function()
        async def get_range_async(start: int, end: int) -> list:
            return list(range(start, end))

        result = await get_range_async(1, 6)
        assert result == [1, 2, 3, 4, 5]

        spans = self.exporter.get_spans_by_name("get_range_async")
        assert len(spans) == 1
        span = spans[0]

        expected_attributes = [
            ("code.function.parameter.start", 1),
            ("code.function.parameter.end", 6),
            ("code.function.return.value", [1, 2, 3, 4, 5]),
        ]
        attributes_match = GenAiTraceVerifier().check_decorator_span_attributes(span, expected_attributes)
        assert attributes_match is True



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import

from ._operations import Int32ValueOperations  # type: ignore
from ._operations import Int64ValueOperations  # type: ignore
from ._operations import BooleanValueOperations  # type: ignore
from ._operations import StringValueOperations  # type: ignore
from ._operations import Float32ValueOperations  # type: ignore
from ._operations import DatetimeValueOperations  # type: ignore
from ._operations import DurationValueOperations  # type: ignore
from ._operations import UnknownValueOperations  # type: ignore
from ._operations import ModelValueOperations  # type: ignore
from ._operations import RecursiveModelValueOperations  # type: ignore
from ._operations import NullableFloatValueOperations  # type: ignore

from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Int32ValueOperations",
    "Int64ValueOperations",
    "BooleanValueOperations",
    "StringValueOperations",
    "Float32ValueOperations",
    "DatetimeValueOperations",
    "DurationValueOperations",
    "UnknownValueOperations",
    "ModelValueOperations",
    "RecursiveModelValueOperations",
    "NullableFloatValueOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class AggregationGranularity(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of AggregationGranularity."""

    DAILY = "Daily"
    """Daily."""
    HOURLY = "Hourly"
    """Hourly."""


class OfferTermInfoName(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Name of the offer term."""

    RECURRING_CHARGE = "Recurring Charge"
    """Recurring Charge."""
    MONETARY_COMMITMENT = "Monetary Commitment"
    """Monetary Commitment."""
    MONETARY_CREDIT = "Monetary Credit"
    """Monetary Credit."""
